<?php

namespace Drupal\webform_submissions_delete\Form;

use Drupal\Core\Url;
use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\WebformInterface;
use Drupal\webform\Form\WebformResultsClearForm;

/**
 * Webform for webform submission purge webform.
 */
class WebformResultsBulkDeleteForm extends WebformResultsClearForm {

  /**
   * {@inheritdoc}
   */
	public function getFormId() {
		return 'results_bulk_delete_form';
	}

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
	$submission_total = $this->submissionStorage->getTotal($this->webform, $this->sourceEntity);
	if($submission_total){
	$form['delete_start_date'] = array(
      '#title'       => t('Start Date'),
      '#type' => 'date',
      '#date_year_range' => '-10:+10',
      '#date_format' => 'Y-m-d',	  
    );
    $form['delete_end_date'] = array(
      '#title'       => t('End Date'),
      '#type' => 'date',
      '#date_year_range' => '-10:+10',
      '#date_date_format' => 'd/m/Y',
    );
    $form['delete_submit'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
    );
	return $form;
  }
  else {
      $t_args = ['%label' => $this->getLabel()];
      $form['message'] = [
        '#type' => 'webform_message',
        '#message_type' => 'error',
        '#message_message' => $this->t('There are no %label submissions.', $t_args),
      ];
      return $form;
  }
  }
  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $start_date = $form_state->getValue('delete_start_date');
    $end_date = $form_state->getValue('delete_end_date');
    if ($start_date == '') {
		$form_state->setErrorByName('delete_start_date', 'Please enter Start Date.');
    }
    if ($end_date == '') {
	  $form_state->setErrorByName('delete_end_date', 'Please enter End Date.');
    }
    if (strtotime($start_date) > strtotime($end_date)) {
      $form_state->setErrorByName('delete_start_date', t('Start date should not greater than End date'));
    }
  }
  
  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
	$start_date = strtotime($form_state->getValue('delete_start_date'));
    $end_date = $form_state->getValue('delete_end_date');  
    if ($end_date != '' && ($end_date = strtotime($end_date)) !== FALSE) {
      if ($end_date == strtotime('midnight', $end_date)) {
        // Full day specified.
        $end_date += 86399;
      }
    }	
	$count = $this->getTotal($start_date, $end_date);
	$batchLimit = $this->getBatchLimit();
	if ($count < $batchLimit) {
      $this->deleteSubmissions($start_date, $end_date);
      $this->messenger()->addStatus($this->getFinishedMessage());
    }
    else {
      $this->batchSet($start_date, $end_date , $count, $batchLimit);
    }
  }
  private function getTotal($startDate, $endDate) {
	$entity_query = $this->submissionStorage->getQuery();
	$entity_query->condition('webform_id', $this->webform->id(), '=');
    $entity_query->condition('created', $startDate, '>=');
    $entity_query->condition('created', $endDate, '<=');
    $result = $entity_query->count()->execute();
	return $result;
  }
  private function deleteSubmissions($startDate, $endDate, $limit = NULL) {
	$entity_query = $this->submissionStorage->getQuery();
	$entity_query->condition('webform_id', $this->webform->id(), '=');
    $entity_query->condition('created', $startDate, '>=');
    $entity_query->condition('created', $endDate, '<=');
	$entity_query->sort('sid');
    if ($limit) {
      $entity_query->range(0, $limit);
    }
    $entity_ids = $entity_query->execute();
    $entities = $this->submissionStorage->loadMultiple($entity_ids);
    $this->submissionStorage->delete($entities);
    return count($entities);
  }
  public function batchSet($start_date, $end_date, $count, $batchLimit) {
	$totalBatch = (int)$count/$batchLimit;
	$reminder = $count%$batchLimit;
	if ($reminder > 0 ) {
		$totalBatch++;
	}
	for($i = 0; $i < $totalBatch ; $i++) {
	$operations[] = array(
      [$this, 'batchProcess'],
      array(
        $start_date,$end_date, $batchLimit,
      ),
    );
	}
    $batch = [
      'title' => $this->t('Clear submissions'),
      'init_message' => $this->t('Clearing submission data'),
      'error_message' => $this->t('The submissions could not be cleared because an error occurred.'),
      'operations' => $operations,
      'finished' => [$this, 'batchFinish'],
    ];
    batch_set($batch);
  }
  public function batchProcess($start_date, $end_date, $batchLimit, &$context) {
	$this->deleteSubmissions($start_date, $end_date, $batchLimit);
	$context['message'] = $this->t('Deleting submissions');
  }
}